@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2011, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2016, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at http://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.haxx.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -24,7 +24,7 @@
 /* The fail macros mark the current test step as failed, and continue */
 #define fail_if(expr, msg)                              \
   if(expr) {                                            \
-    fprintf(stderr, "%s:%d Assertion '%s' met: %s\n" ,  \
+    fprintf(stderr, "%s:%d Assertion '%s' met: %s\n",   \
             __FILE__, __LINE__, #expr, msg);            \
     unitfail++;                                         \
   }
@@ -36,11 +36,14 @@
     unitfail++;                                          \
   }
 
-#define verify_memory(dynamic, check, len)                              \
-  if(dynamic && memcmp(dynamic, check, len)) {                          \
-    fprintf(stderr, "%s:%d The dynamic string didn't match '%s'\n",     \
-            __FILE__, __LINE__, check);                                 \
-    unitfail++;                                                         \
+#define verify_memory(dynamic, check, len)                                  \
+  if(dynamic && memcmp(dynamic, check, len)) {                              \
+    fprintf(stderr, "%s:%d Memory buffer mismatch size %d. '%s' is not\n",  \
+            __FILE__, __LINE__, len,                                        \
+            hexdump((const unsigned char *)check, len));                    \
+    fprintf(stderr, "%s:%d the same as '%s'\n", __FILE__, __LINE__,         \
+            hexdump((const unsigned char *)dynamic, len));                  \
+    unitfail++;                                                             \
   }
 
 /* fail() is for when the test case figured out by itself that a check
@@ -55,7 +58,7 @@
 /* The abort macros mark the current test step as failed, and exit the test */
 #define abort_if(expr, msg)                                   \
   if(expr) {                                                  \
-    fprintf(stderr, "%s:%d Abort assertion '%s' met: %s\n" ,  \
+    fprintf(stderr, "%s:%d Abort assertion '%s' met: %s\n",   \
             __FILE__, __LINE__, #expr, msg);                  \
     unitfail++;                                               \
     goto unit_test_abort;                                     \
@@ -83,10 +86,11 @@ extern int unitfail;
 #define UNITTEST_START                          \
   int test(char *arg)                           \
   {                                             \
-  (void)arg;                                    \
-  if (unit_setup()) {                           \
-    fail("unit_setup() failure");               \
-  } else {
+    (void)arg;                                  \
+    if(unit_setup()) {                          \
+      fail("unit_setup() failure");             \
+    }                                           \
+    else {
 
 #define UNITTEST_STOP                           \
     goto unit_test_abort; /* avoid warning */   \
@@ -95,4 +99,3 @@ unit_test_abort:                                \
   }                                             \
   return unitfail;                              \
   }
-
